Package: salmonMSE
Title: Management Strategy Evaluation for Salmon Species
Version: 2.1.0
Authors@R: 
    person("Quang", "Huynh", , "quang@bluematterscience.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-7835-4376"))
Date: 2026-04-20
Maintainer: Quang Huynh <quang@bluematterscience.com>
Description: Simulation tools to evaluate the long-term effects of salmon management strategies, including a combination of habitat, harvest, and
  habitat actions. The stochastic age-structured operating model accommodates complex life histories, including freshwater survival across 
  early life stages, juvenile survival and fishery exploitation in the marine life stage, partial maturity by age class, and fitness impacts of 
  hatchery programs on natural spawning populations. 'salmonMSE' also provides an age-structured conditioning model to develop operating models 
  fitted to data.
License: GPL (>= 3)
Depends: R (>= 3.5.0)
Imports: abind, MSEtool (>= 3.7.2), RTMB (>= 1.9), dplyr, ggplot2,
        grDevices, gsl, methods, reshape2, rlang, rmarkdown, stats,
        utils
Suggests: ggrepel, rstan, scales, testthat (>= 3.0.0), tmbstan
Encoding: UTF-8
LazyData: yes
LazyLoad: yes
RoxygenNote: 7.3.3
URL: https://docs.salmonmse.com/,
        https://github.com/Blue-Matter/salmonMSE
BugReports: https://github.com/Blue-Matter/salmonMSE/issues
NeedsCompilation: no
Packaged: 2026-04-24 22:04:44 UTC; quang
Author: Quang Huynh [aut, cre] (ORCID: <https://orcid.org/0000-0001-7835-4376>)
Repository: CRAN
Date/Publication: 2026-04-25 09:40:13 UTC
Built: R 4.5.2; ; 2026-05-01 18:29:27 UTC; unix
