# Generated by roxygen2: do not edit by hand

export(AHA)
export(CM2SOM)
export(CM_CWT_ER)
export(CM_ER)
export(CM_F)
export(CM_M)
export(CM_MSY)
export(CM_Megg)
export(CM_Njuv)
export(CM_SRR)
export(CM_Sgen)
export(CM_Srep)
export(CM_covariate)
export(CM_esc)
export(CM_fit_CWTcatch)
export(CM_fit_CWTesc)
export(CM_fit_esc)
export(CM_fit_pHOS)
export(CM_int)
export(CM_maturity)
export(CM_pairs)
export(CM_prod)
export(CM_recr)
export(CM_surv)
export(CM_surv2)
export(CM_trace)
export(CM_vul)
export(CM_wt)
export(CM_wto)
export(PNI50)
export(PNI80)
export(P_PNI50)
export(P_PNI80)
export(P_SMSY80)
export(P_SMSY85)
export(P_Sgen100)
export(P_WILD50)
export(ProjectSOM)
export(SMSY85)
export(SOM)
export(Sgen100)
export(WILD50)
export(calc_MSY)
export(calc_SRR)
export(calc_SRRpars)
export(calc_Sgen)
export(calc_Sgen_Ricker)
export(calc_Smsy_Ricker)
export(calc_Umsy_Ricker)
export(calc_pwild)
export(calc_ref)
export(calc_smolt)
export(check_SOM)
export(compare)
export(compare_escapement)
export(compare_fitness)
export(compare_spawners)
export(compare_statevar_hist)
export(compare_statevar_ts)
export(fit_CM)
export(get_CMdata)
export(get_report)
export(plot_Kobe)
export(plot_LHG)
export(plot_RS)
export(plot_decision_table)
export(plot_decision_table2)
export(plot_escapement)
export(plot_fishery)
export(plot_fitness)
export(plot_spawners)
export(plot_statevar_hist)
export(plot_statevar_ts)
export(plot_tradeoff)
export(report)
export(report_CM)
export(salmonMSE)
export(salmonMSE_env)
export(sample_CM)
export(simpleSOM)
export(simple_salmonMSE)
exportClasses(Bio)
exportClasses(Habitat)
exportClasses(Harvest)
exportClasses(Hatchery)
exportClasses(Historical)
exportClasses(SMSE)
exportClasses(SOM)
exportClasses(simpleSOM)
exportMethods(report)
import(RTMB)
import(ggplot2)
importFrom(abind,abind)
importFrom(dplyr,"%>%")
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,rainbow)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,box)
importFrom(graphics,grid)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matlines)
importFrom(graphics,matplot)
importFrom(graphics,pairs)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(gsl,lambert_W0)
importFrom(methods,"slot<-")
importFrom(methods,.hasSlot)
importFrom(methods,.slotNames)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,parLapplyLB)
importFrom(parallel,parSapplyLB)
importFrom(parallel,stopCluster)
importFrom(rlang,.data)
importFrom(rlang,.env)
importFrom(rmarkdown,render)
importFrom(stats,cor)
importFrom(stats,density)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,nlminb)
importFrom(stats,optimize)
importFrom(stats,quantile)
importFrom(stats,rlnorm)
importFrom(stats,sd)
importFrom(utils,browseURL)
importFrom(utils,packageVersion)
