

#' Example objects from salmonMSE
#'
#' @description
#' S4 objects containing operating model inputs to run the projection.
#' @format An object of class [salmonMSE::SOM-class].
#' @source See salmonMSE [website](https://docs.salmonmse.com/articles/example.html)
#'
#' @aliases multi_SOM
#' @examples
#' data(simple_SOM)
#'
#' data(multi_SOM)
"simple_SOM"

#' @rdname simple_SOM
"multi_SOM"
