
---
title: "salmonMSE projection"
date: "`r Sys.Date()`"
output:
  html_document:
    highlight: rstudio
    df_print: paged
---

<style type="text/css">
h1 { /* Header 1 */
  font-size: 24px;
}
</style>

```{r setup, include = FALSE, echo = FALSE}
library(dplyr)
knitr::opts_chunk$set(
  collapse = TRUE, echo = FALSE, message = FALSE,
  fig.width = 6, fig.height = 4.5, out.width = "650px", comment = "#>"
)
```

# NAME {.tabset}

<!-- ADD RMD BY STOCK -->

## About

This report was generated on: `r Sys.time()`<br />
salmonMSE version `r packageVersion("salmonMSE")`<br />
`r R.version.string`
