% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{glossary}
\alias{glossary}
\title{salmonMSE glossary}
\description{
Glossary of terms and parameters used in salmonMSE
}
\examples{
data(glossary)
glossary[1:2, ]
}
